﻿
using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

/*             Digital image processing: Create pencil drawing              */
/*                           www.mosismath.com                              */

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {

        BackgroundWorker calculationThread;
        Bitmap bmpSource;
        Bitmap bmpDest;

        public MainWin()
        {
            InitializeComponent();
            // Set the initial size of the actual image for scrolling
            pbSource.Height = 2112;
            pbSource.Width = 2816;
            pbDest.Height = 2112;
            pbDest.Width = 2816;

            calculationThread = new BackgroundWorker();
            calculationThread.DoWork += calculate;
            calculationThread.RunWorkerCompleted += calculationCompleted;
        }

        private Bitmap ProcessImage(Bitmap bmpSource, double[,] s, int order, int threshold)
        {
            int x, y, i, j;
            double R, G, B;
            Bitmap bmpDest = new Bitmap(pbDest.Image);
            for (x = 1; x < bmpSource.Width - 1; x++)
            {
                for (y = 1; y < bmpSource.Height - 1; y++)
                {
                    R = 0;
                    G = 0;
                    B = 0;
                    for (i = 0; i < order; i++)
                    {
                        for (j = 0; j < order; j++)
                        {
                            Color pix = bmpSource.GetPixel(x - 1 + i, y - 1 + j);
                            R += (pix.R * s[i, j]);
                            G += (pix.G * s[i, j]);
                            B += (pix.B * s[i, j]);
                        }
                    }
                    if (R < 0)
                        R = 0;
                    else
                    {
                        if (R > 255)
                            R = 255;
                    }
                    if (G < 0)
                        G = 0;
                    else
                    {
                        if (G > 255)
                            G = 255;
                    }
                    if (B < 0)
                        B = 0;
                    else
                    {
                        if (B > 255)
                            B = 255;
                    }
                    if(Math.Sqrt(R*R + G*G + B*B) > threshold)
                    {
                        R = 255;
                        G = 255;
                        B = 255;
                    }
                    else
                    {
                        R = 0;
                        G = 0;
                        B = 0;
                    }
                    Color newPix = Color.FromArgb((int)(255 - R), (int)(255 - G), (int)(255 - B));
                    bmpDest.SetPixel(x, y, newPix);
                }
            }
            return bmpDest;
        }

        private void calculate(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            int threshold = 120;
            double[,] s = { { -3, -3, -3 }, { -3, 24, -3 }, { -3, -3, -3 } };
            byte[] buffer;
            // Read image file
            FileStream fileStream = new FileStream(Application.StartupPath + "\\IMG_Monterosa.JPG", FileMode.Open, FileAccess.Read);
            try
            {
                int length = (int)fileStream.Length;
                buffer = new byte[length];
                int count;
                int sum = 0;

                while ((count = fileStream.Read(buffer, sum, length - sum)) > 0)
                    sum += count; 
            }
            finally
            {
                fileStream.Close();
            }
            MemoryStream imageStream = new MemoryStream(buffer, 0, buffer.Length);
            string tempString = System.Text.Encoding.ASCII.GetString(buffer);
            char[] bufferJpeg = new char[tempString.Length];
            tempString.CopyTo(0, bufferJpeg, 0, tempString.Length);
            imageStream.Position = 0;
            bmpSource = new Bitmap(imageStream);
            bmpDest = new Bitmap(pbDest.Image);

            bmpDest = ProcessImage(bmpSource, s, 3, threshold);
        }


        void calculationCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            pbDest.Image = bmpDest;
            pbDest.Image.Save(Application.StartupPath + "\\Monterosa_gr.jpg");
            pbDest.Width = bmpDest.Width;
            pbDest.Height = bmpDest.Height;
            btProc.Enabled = true;
            Cursor.Current = Cursors.Default;
            this.Cursor = Cursors.Default;
        }


        private void btProc_Click(object sender, EventArgs e)
        {
            btProc.Enabled = false;
            this.Cursor = Cursors.WaitCursor;
            Cursor.Current = Cursors.WaitCursor;
            calculationThread.RunWorkerAsync();
        }
    }
}
